/*---------------------------------------------------------------------------\
| Project:          Generic LV2 PIC18F458 CAN node firmware
|
| File:             ports.c
|
| Description:      Code which deals with the PIC18F458's I/O ports.
|
| Global Routines:  InitializePorts()
|
| Local Routines:   None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/



//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <piccore.h>   // Includes for the project (includes other includes!)



/*---------------------------------------------------------------------------\
| Function:     InitializePorts
|
| Purpose:      Initialize the PIC's ports out of reset.
|
| Discussion:   Huh?
|
| Expects:      That the ADC is setup later - e.g., that you want all
|               digital I/O for now.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void InitializePorts ( uint8_t port_a_latch,    uint8_t port_a_direction,
                       uint8_t port_b_latch,    uint8_t port_b_direction,
                       uint8_t port_c_latch,    uint8_t port_c_direction,
                       uint8_t port_d_latch,    uint8_t port_d_direction,
                       uint8_t port_e_latch,    uint8_t port_e_direction,
                       uint8_t port_options )
{

    //------------------------------------------------------------------------
    // PORTA: RA0 - RA5 (CAUTION: ADC ADCON1 OVERRIDES TRISA, RA5 INPUT ONLY)
    //------------------------------------------------------------------------

    LATA = port_a_latch;
    TRISA = port_a_direction;

    //------------------------------------------------------------------------
    // PORTB: RB0 - RB7 (CAUTION: ICD USES RB6,RB7, CAN USES RB2, RB3)
    //------------------------------------------------------------------------

    // PORTB PIN 3 (RB3) MUST BE SET TO ZERO!!!! OTHERWISE, A ONE WRITTEN
    // TO THE CAN BUS WILL BRING DOWN THE WHOLE DAMN BUS. WE'lL FIX THIS
    // POSSIBILITY IN THE NEXT ROUND OF HARDWARE - ADD A HIGH PASS FILTER
    // TO THE OUTPUT BETWEEN THE PIC AND THE TRANSCIEVER SO THAT
    // AFTER A SECOND OR SO THE OUTPUT IS ATTENUATED TO ZERO.

    if (port_options & PORTS_PORTB_PULLUP_MASK )
        RBPU = 0;
    else
        RBPU = 1;

    LATB = port_b_latch;

    TRISB = port_b_direction;

    // Select the rising or falling edge of RB0/1 to trigger INT0/1

    if (port_options & PORTS_INT0_EDGE_MASK)
        INTEDG0 = 1;
    else
        INTEDG0 = 0;

    if (port_options & PORTS_INT1_EDGE_MASK)
        INTEDG1 = 1;
    else
        INTEDG1 = 0;

    // Set the interrupt priority on Int1 (on PORTB) interrupts
    // (Note that INT0 interrupts must be high priority

    #if INTS_INT1_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    Int1PrioritySetting = HIGH_PRIORITY_INT;
    #else
    Int1PrioritySetting = LOW_PRIORITY_INT;
    #endif

    if (port_options & PORTS_INT0_INTS_MASK)
        enable_int0_ints();
    else
        disable_int0_ints();

    if (port_options & PORTS_INT1_INTS_MASK)
        enable_int1_ints();
    else
        disable_int1_ints();

    //------------------------------------------------------------------------
    // PORTC: RC0 - RC7 (Caution: serial)
    //------------------------------------------------------------------------

    LATC = port_c_latch;
    TRISC = port_c_direction;

    //------------------------------------------------------------------------
    // PORTD: RD0 - RD7
    //------------------------------------------------------------------------

    LATD = port_d_latch;
    TRISD = port_d_direction;

    //------------------------------------------------------------------------
    // PORTE: RE0 - RE2 (CAUTION: ADC ADCON1 OVERRIDES TRISE)
    //------------------------------------------------------------------------

    LATE = port_e_latch;
    TRISE = port_e_direction;

}




/*---------------------------------------------------------------------------\
| Function:     int0_isr
|
| Purpose:      Interrupt Service Routine for the INT0 interrupt off of RB0
|
| Discussion:   Huh?
|
| Expects:
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void int0_isr (void)
{
    // Clear the interrupt

    clear_int0_interrupt();

    #if INTS_INT0_APPLICATION_HOOK == INTS_HOOK_ENABLED
    int0_application_handler();
    #endif
}





/*---------------------------------------------------------------------------\
| Function:     int1_isr
|
| Purpose:      Interrupt Service Routine for the INT1 interrupt off of RB1
|
| Discussion:   Huh?
|
| Expects:
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void int1_isr (void)
{
    // Clear the interrupt

    clear_int1_interrupt();

    #if INTS_INT1_APPLICATION_HOOK == INTS_HOOK_ENABLED
    int1_application_handler();
    #endif
}


/*---------------------------------------------------------------------------\
| Function:     port_b_ioc_isr
|
| Purpose:      Interrupt Service Routine for the Port B Interrupt On Change
|
| Discussion:   Huh?
|
| Expects:
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void port_b_ioc_isr (void)
{
    // Clear the interrupt

    clear_port_b_ioc_interrupt();

    #if INTS_PBIOC_APPLICATION_HOOK == INTS_HOOK_ENABLED
    port_b_ioc_application_handler();
    #endif
}





